@@ -41,17 +41,16 @@ bool Rotator::initProperties()
     RotatorInterface::initProperties(MAIN_CONTROL_TAB);
 
     // Presets
-    IUFillNumber(&PresetN[0], "PRESET_1", "Preset 1", "%.f", 0, 360, 10, 0);
-    IUFillNumber(&PresetN[1], "PRESET_2", "Preset 2", "%.f", 0, 360, 10, 0);
-    IUFillNumber(&PresetN[2], "PRESET_3", "Preset 3", "%.f", 0, 360, 10, 0);
-    IUFillNumberVector(&PresetNP, PresetN, 3, getDeviceName(), "Presets", "", "Presets", IP_RW, 0, IPS_IDLE);
+    PresetNP[0].fill("PRESET_1", "Preset 1", "%.f", 0, 360, 10, 0);
+    PresetNP[1].fill("PRESET_2", "Preset 2", "%.f", 0, 360, 10, 0);
+    PresetNP[2].fill("PRESET_3", "Preset 3", "%.f", 0, 360, 10, 0);
+    PresetNP.fill(getDeviceName(), "Presets", "", "Presets", IP_RW, 0, IPS_IDLE);
 
     //Preset GOTO
-    IUFillSwitch(&PresetGotoS[0], "Preset 1", "", ISS_OFF);
-    IUFillSwitch(&PresetGotoS[1], "Preset 2", "", ISS_OFF);
-    IUFillSwitch(&PresetGotoS[2], "Preset 3", "", ISS_OFF);
-    IUFillSwitchVector(&PresetGotoSP, PresetGotoS, 3, getDeviceName(), "Goto", "", "Presets", IP_RW, ISR_1OFMANY, 0,
-                       IPS_IDLE);
+    PresetGotoSP[0].fill("Preset 1", "", ISS_OFF);
+    PresetGotoSP[1].fill("Preset 2", "", ISS_OFF);
+    PresetGotoSP[2].fill("Preset 3", "", ISS_OFF);
+    PresetGotoSP.fill(getDeviceName(), "Goto", "", "Presets", IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
     addDebugControl();
 
@@ -101,13 +100,13 @@ bool Rotator::updateProperties()
 
     if (isConnected())
     {
-        defineProperty(&PresetNP);
-        defineProperty(&PresetGotoSP);
+        defineProperty(PresetNP);
+        defineProperty(PresetGotoSP);
     }
     else
     {
-        deleteProperty(PresetNP.name);
-        deleteProperty(PresetGotoSP.name);
+        deleteProperty(PresetNP);
+        deleteProperty(PresetGotoSP);
     }
 
     return true;
@@ -115,22 +114,18 @@ bool Rotator::updateProperties()
 
 bool Rotator::ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
 {
+    if (RotatorInterface::processNumber(dev, name, values, names, n))
+        return true;
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-        if (!strcmp(name, PresetNP.name))
+        if (PresetNP.isNameMatch(name))
         {
-            IUUpdateNumber(&PresetNP, values, names, n);
-            PresetNP.s = IPS_OK;
-            IDSetNumber(&PresetNP, nullptr);
+            PresetNP.update(values, names, n);
+            PresetNP.setState(IPS_OK);
+            PresetNP.apply();
 
             return true;
         }
-
-        if (strstr(name, "ROTATOR"))
-        {
-            if (RotatorInterface::processNumber(dev, name, values, names, n))
-                return true;
-        }
     }
 
     return DefaultDevice::ISNewNumber(dev, name, values, names, n);
@@ -138,35 +133,31 @@ bool Rotator::ISNewNumber(const char *dev, const char *name, double values[], ch
 
 bool Rotator::ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
 {
+    if (RotatorInterface::processSwitch(dev, name, states, names, n))
+        return true;
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-        if (!strcmp(PresetGotoSP.name, name))
+        if (PresetGotoSP.isNameMatch(name))
         {
-            IUUpdateSwitch(&PresetGotoSP, states, names, n);
-            int index = IUFindOnSwitchIndex(&PresetGotoSP);
+            PresetGotoSP.update(states, names, n);
+            int index = PresetGotoSP.findOnSwitchIndex();
 
-            if (MoveRotator(PresetN[index].value) != IPS_ALERT)
+            if (MoveRotator(PresetNP[index].getValue()) != IPS_ALERT)
             {
                 // Ensure Goto Rotator is set to busy.
-                GotoRotatorNP.s = IPS_BUSY;
-                IDSetNumber(&GotoRotatorNP, nullptr);
+                GotoRotatorNP.setState(IPS_BUSY);
+                GotoRotatorNP.apply();
 
-                PresetGotoSP.s = IPS_OK;
-                DEBUGF(Logger::DBG_SESSION, "Moving to Preset %d with angle %g degrees.", index + 1, PresetN[index].value);
-                IDSetSwitch(&PresetGotoSP, nullptr);
+                PresetGotoSP.setState(IPS_OK);
+                DEBUGF(Logger::DBG_SESSION, "Moving to Preset %d with angle %g degrees.", index + 1, PresetNP[index].getValue());
+                PresetGotoSP.apply();
                 return true;
             }
 
-            PresetGotoSP.s = IPS_ALERT;
-            IDSetSwitch(&PresetGotoSP, nullptr);
+            PresetGotoSP.setState(IPS_ALERT);
+            PresetGotoSP.apply();
             return false;
         }
-
-        if (strstr(name, "ROTATOR"))
-        {
-            if (RotatorInterface::processSwitch(dev, name, states, names, n))
-                return true;
-        }
     }
 
     return DefaultDevice::ISNewSwitch(dev, name, states, names, n);
@@ -182,8 +173,8 @@ bool Rotator::saveConfigItems(FILE *fp)
     DefaultDevice::saveConfigItems(fp);
     RI::saveConfigItems(fp);
 
-    IUSaveConfigNumber(fp, &PresetNP);
-    IUSaveConfigSwitch(fp, &ReverseRotatorSP);
+    PresetNP.save(fp);
+    ReverseRotatorSP.save(fp);
 
     return true;
 }
